/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "engine/aienginepluginmanager.h"
#include "engineselector/engineselector.h"
#include "server/servermanager.h"
#include "threadpool/async.h"

int main(int argc, char *argv[]) {
    try {
        logger::init();
        cpr::async::startup(8);

        ai_engine::AiEnginePluginManager aiEnginePluginManager;
        ModelConfigManager modelConfigManager(aiEnginePluginManager);
        EngineSelector engineSelector(aiEnginePluginManager,
                                      modelConfigManager);

        ServerManager serverManager(aiEnginePluginManager, engineSelector,
                                    modelConfigManager);

        serverManager.startServers();
    } catch (std::exception &exception) {
        LOGE("exception: ", exception.what());
    }
    return 0;
}
