/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef RAGSERVERPROXY_H
#define RAGSERVERPROXY_H

#include <jsoncpp/json/json.h>

#include <string>
#include <vector>

struct RagErrorInfo {
    int32_t errorCode;
    std::string errorMessage;
};

class RagServerProxy {
public:
    static RagServerProxy &instance();

    bool isValid() const;
    RagServerProxy(const RagServerProxy &) = delete;
    RagServerProxy &operator=(const RagServerProxy &) = delete;

    std::string createKnowledgeBase();
    void deleteKnowledgeBase(const std::string &knowledgeBaseName);
    RagErrorInfo addFiles(const std::string &knowledgeBaseName,
                          const std::vector<std::string> &files);
    // <filepath, content>
    using FileContentTupleVector =
        std::vector<std::tuple<std::string, std::string>>;
    FileContentTupleVector similaritySearch(
        const std::string &knowledgeBaseName,
        const std::string &searchConditions);

    std::string getFileContent(const std::string &knowledgeBaseName,
                               const std::string &filePath);

private:
    RagServerProxy();
    FileContentTupleVector handleSearchResult(const Json::Value &jsonResult);
};

#endif  // RAGSERVERPROXY_H
